@@ -5,6 +5,7 @@ import asyncio
 import websockets
 import json
 import urllib
+import threading
 from dataclasses import asdict
 from aiohttp import web
 from rpc_gateway import messages, errors
@@ -59,7 +60,7 @@ class Gateway:
         self.logger.info(f'Starting on ws://{self.host}:{self.port}')
         self.websocket = await websockets.serve(self.on_connection, self.host, self.port)
 
-        self.logger.info(f'Starting HTTP server on http://{self.host}:{self.http_port}')
+        self.logger.info(f'Starting HTTP server on http://{self.host}:{self.http_port}, auth_key={self.auth_key}')
         self.http_server = web.Server(self.handle_http_request)
         self.http_runner = web.ServerRunner(self.http_server)
         await self.http_runner.setup()
@@ -114,6 +115,12 @@ class Gateway:
         server = self.instances[instance_name]
         return await server.request(messages.Request(method=messages.Method.UNLOCK, data=instance_name), raise_error=False)
 
+    def is_registered(self, group: str, instance: str) -> bool:
+        if group not in self.instance_groups:
+            return False
+
+        return instance in self.instance_groups[group]
+
     #
     # Request Handlers
     #
@@ -155,6 +162,11 @@ class Gateway:
     async def handle_register_request(self, websocket_connection: WebsocketConnection, request: messages.Request) -> messages.Response:
         self.logger.info(f'Registering instances: {request.data}')
 
+        # check to see if this instance
+        for instance_name, instance_group in request.data:
+            if self.is_registered(instance_group, instance_name):
+                return messages.Response(status=messages.Status.ERROR, data=errors.InstanceAlreadyRegisteredError(f'Instance "{instance_name}" already registered with "{instance_group}" group'))
+
         for instance_name, instance_group in request.data:
             if instance_group not in self.instance_groups:
                 self.instance_groups[instance_group] = []
@@ -264,6 +276,7 @@ class GatewayClient:
         self.websocket_connection = WebsocketConnection(request_handler=self._log_and_handle_request, close_handler=self._handle_close)
         self.event_loop = asyncio.get_event_loop()
         self.connect_retry_timeout = 2.0
+        self.start_thread = threading.Thread(name='RPC gateway client', target=self._start_sync, daemon=True)
 
     @property
     def connected(self) -> bool:
@@ -280,18 +293,21 @@ class GatewayClient:
                 time.sleep(self.connect_retry_timeout)
 
     def start(self, wait=True):
-        await_sync(self._start(wait), self.event_loop)
+        self.start_thread.start()
+        if wait:
+            self.start_thread.join()
 
-    async def _start(self, wait=True):
+    def _start_sync(self):
+        await_sync(self._start(), self.event_loop)
+
+    async def _start(self):
         await self._connect()
         await self.websocket_connection.start(wait=False, connection=self.connection)
         await self._on_start()
-
-        if wait:
-            await self._wait()
+        await self._wait()
 
     def wait(self):
-        await_sync(self._wait(), self.event_loop)
+        self.start_thread.join()
 
     async def _wait(self):
         await self.websocket_connection.wait()
@@ -301,6 +317,7 @@ class GatewayClient:
 
     def stop(self):
         await_sync(self._stop(), self.event_loop)
+        self.start_thread.join()
 
     async def _stop(self):
         await self.websocket_connection.stop()
