@@ -38,6 +38,7 @@ def is_json_serializable(value: Any) -> bool:
 class Instance:
     name: str
     instance: Any
+    group: str
     locked: bool
 
 
@@ -124,24 +125,28 @@ class InstanceMetadata:
 
 
 class Server(gateway.GatewayClient):
-    def __init__(self, gateway_url: str = 'ws://localhost:8888', max_workers: int = 5, instances: Optional[Dict[str, Any]] = None):
-        super().__init__(gateway_url)
+    def __init__(self, gateway_url: str = 'ws://localhost:8888', max_workers: int = 5, instances: Optional[Dict[str, Instance]] = None, auth_key: str = 'DEFAULT_KEY'):
+        super().__init__(gateway_url=gateway_url, auth_key=auth_key)
         self.instances = {} if instances is None else instances
         self.executor = ThreadPoolExecutor(max_workers=max_workers)
 
     async def _on_start(self):
-        await self._register_gateway_instances(list(self.instances.keys()))
+        await self._register_gateway_instances(self.instances)
 
-    async def _register_gateway_instances(self, instances: List[str]):
-        await self.websocket_connection.send_request(messages.Method.REGISTER, instances)
+    async def _register_gateway_instances(self, instances: Dict[str, Instance]):
+        instance_data = [(instance.name, instance.group) for name, instance in instances.items()]
+        await self.websocket_connection.send_request(messages.Method.REGISTER, instance_data)
 
-    def register(self, name: str, instance: Any) -> Any:
+    def register(self, name: str, instance: Any, group: Optional[str] = None) -> Any:
         self.logger.info(f'Registering instance: {name}')
 
         if name in self.instances:
             raise errors.InstanceAlreadyRegisteredError(f'Instance already registered with name: {name}')
 
-        self.instances[name] = Instance(name=name, instance=instance, locked=False)
+        if group is None:
+            group = instance.__class__.__name__
+
+        self.instances[name] = Instance(name=name, instance=instance, locked=False, group=group)
 
         return instance
 
