@@ -1,6 +1,7 @@
 from typing import Optional, Dict, Any, Callable, Awaitable
 import asyncio
 import pickle
+import json
 import logging
 import websockets
 from concurrent import futures
@@ -52,8 +53,11 @@ class WebsocketConnection:
             await self.wait()
 
     async def wait(self):
-        await self.message_send_task
-        await self.message_receive_task
+        try:
+            await self.message_send_task
+            await self.message_receive_task
+        except futures.CancelledError:
+            pass
 
     async def stop(self):
         self.running = False
@@ -118,15 +122,18 @@ class WebsocketConnection:
 
     async def _handle_request(self, request: messages.Request):
         id = request.id
+        encoding = request.encoding
         response = await self.request_handler(self, request)
         response.id = id
+        response.encoding = encoding
         await self.send_message(response)
 
     async def _message_send(self):
         while self.running:
             message: messages.Message = await self.send_queue.get()
+
             try:
-                message_bytes = message.to_pickle()
+                message_bytes = messages.encode(message)
             except (pickle.PickleError, AttributeError):
                 message_bytes = messages.Response(id=message.id, status=messages.Status.ERROR, data=errors.SerializationError(f'Cannot serialize response data')).to_pickle()
 
@@ -138,7 +145,7 @@ class WebsocketConnection:
             while self.running:
                 message_bytes = await self.connection.recv()
                 try:
-                    message = pickle.loads(message_bytes)
+                    message = messages.decode(message_bytes)
                 except (pickle.UnpicklingError, TypeError):
                     self.logger.error(f'Invalid message: {message_bytes}')
                     continue
